function [] = main() 
% MAIN()
%
% Subject: Project RCP209 @ CNAM Paris, France - Summer 2010
% To:  Michel CRUCIANU <Michel.Crucianu@cnam.fr>, 
%      Meziane YACOUB <meziane.yacoub@cnam.fr>
% 
% ----
% Franck DERNONCOURT <franck.dernoncourt@gmail.com>
% Phone: +33 601 188 949
% GTalk / Skype: franck.dernoncourt
% Address: 75013 Paris, France
% http://francky.me/
% http://www.linkedin.com/in/franckdernoncourt
% MATLAB : R2010a 64-bit (7.10.0.499)
% ----
%
% Data source: http://www.stats.uwo.ca/faculty/aim/epubs/mhsets/thompsto/
% Subject: 
% << 
% Prdiction  du dbit entrant d'un lac. L'objectif est de
% prdire l'volution du dbit entrant d'un lac  partir de
% l'historique de ce dbit, de la fonte des neiges et des prcipitations
% dans le bassin hydrographique.  Donnes :
% http://www.stats.uwo.ca/faculty/aim/epubs/mhsets/thompsto/
% Contact : Michel CRUCIANU <Michel.Crucianu@cnam.fr> 
% >>
%


%% Import utils library
% http://home.online.no/~pjacklam/matlab/software/util/index.html
warning off    % To avoid the 2 warnings that will appear during the import
               % 1) isscalar.m has the same name as a MATLAB builtin. We 
               %    suggest you rename the function to avoid a potential
               %    name conflict. 
               % 2) isvector.m has the same name as a MATLAB builtin. We 
               %    suggest you rename the function to avoid a potential
               %    name conflict. 
addpath('utils'); 
run('utils/addutils.m');
warning on

%% Import the THOMPSTONE data
% 1.  lacstjin.1  --> Lac St-Jean Reservoir, quarter-monthly inflows, 1953-82                        
% 2.  lacstjra.1  --> Lac St-Jean Region, quarter-monthly rainfall, 1953-82                          
% 3.  lacstjsn.1  --> Fonte de neige par quart de mois, 1953-82, Bassin Daval  
inflowsTemp = load('lacstjin.1'); 
rainfallTemp = load('lacstjra.1'); 
snowTemp = load('lacstjsn.1'); 

% lacstjin.1, lacstjra.1 and lacstjsn.1 have to be read left to right, downwards
% whereas the MATLAB load function read them downwards, left to right.
% Therefore, we need to reshape the matrices, otherwise we'll have
% invalid results.
inflows = reshape(inflowsTemp', numel(inflowsTemp), 1);
rainfall = reshape(rainfallTemp', numel(rainfallTemp), 1);
snow = reshape(snowTemp', numel(snowTemp), 1); 
% Note: snow is snowmelt in short, but actually it turns out it would have
%       better to keep snowmelt, as snow is misleading. This needs to be
%       refactor.



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%% PART 1 - DATA ANALYSIS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Basic information
fprintf('\n\n********************************************************\n');
fprintf('Basic information on data\n\n');
disp('****** Inflows summary *****');
showRandomVariableInformation(inflows);
disp('****** Rainfall summary *****');
showRandomVariableInformation(rainfall);
disp('****** Snowmelt summary *****');
showRandomVariableInformation(snow);

%% Box plots
figure('Name','Inflow box plot window','NumberTitle','off')
boxplot(inflows, 'orientation','horizontal', 'labels', ['inflow'])
figure('Name','Rainfall box plot window','NumberTitle','off')
boxplot(horzcat(rainfall, snow), 'orientation','horizontal','labels', ['rainfall';'snowmelt'])

%% Plot rainfall & snow curves
figure('Name','Rainfall & snowmelt curves','NumberTitle','off')
x = 1953:1/48:1983-1/48; % We take 48 since we have 48 values for each year
plot(x,rainfall, 'r')
hold on % When we set hold to on, MATLAB does not remove the existing graph;
        % it adds the new data to the current graph, rescaling if the new 
        % data falls outside the range of the previous axis limits. 
grid on 
plot(x,snow, 'b')
legend(': Rainfall',': Snowmelt');
hold off

%% Plot inflow curve
figure('Name','Inflow curve','NumberTitle','off')
x = 1953:1/48:1983-1/48; % We take 48 since we have 48 values for each year
plot(x,inflows, 'b')
grid on 
legend(': Inflow');


%% Correlation matrix using corrmat.m (from utils library) and corrmap.m 
fprintf('\n');
disp('Correlation matrix:');
disp(corrmat(horzcat(rainfall, inflows, snow)));
figure('Name','Correlation matrix','NumberTitle','off')
corrmapLabels = ['inflows '; 'rainfall'; 'snowmelt'];
corrmap(horzcat(inflows, rainfall, snow), corrmapLabels);


%% Principal Component Analysis (PCA)
% Documentation: % http://www.mathworks.com/access/helpdesk/help/toolbox/stats/brkgqnt.html#f75476

fprintf('\n\n********************************************************\n');
fprintf('Principal Component Analysis (PCA)\n');

% We standardize the data by dividing each column by its standard deviation.
data = horzcat(inflows, rainfall, snow);
stdr = std(data);
sr = data./repmat(stdr,1440,1);

% We compute the PCA
[coefs,scores,variances,t2] = princomp(sr);
coefs,variances

% We use the pareto function to make a scree plot of the percent variability
% explained by each principal component.
figure('Name','Percent variability explained by each principal component','NumberTitle','off')
percent_explained = 100*cumsum(variances)./sum(variances)
pareto(percent_explained)
xlabel('Principal Component')
ylabel('Variance Explained (%)')

% We visualize the Results
figure('Name','Principal Component Analysis','NumberTitle','off')
biplot(coefs(:,1:2),'Scores',scores(:,1:2),'VarLabels', {'Inflows' 'Rainfall' 'Snowmelt' })



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%% PART 2 - DATA LEARNING %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% Save variables in .mat files for later use (autocorrelation)
save('inflows.mat', 'inflows');
save('rainfall.mat', 'rainfall');
save('snow.mat', 'snow');

%% Autocorrelation
% Run ./tsa/tsademo.m to see the autocorrelations graph

% Compute the Average Relative Variance (ARG)
inflowsAutocorrelativlyPredicted = inflows(end-48:end); % We remove the first 48 values

% Computing ARV
numerator = ((sum((inflows(49:1440) - inflows(1:end-48)) .^2)) / size(inflows(49:1440),1));
denominator = ((sum((inflows(49:1440) - mean(inflows(49:1440))) .^2)) / size(inflows(49:1440),1));
AutocorrelationARV = numerator / denominator


%% Using average inflows/rainfall/snow variations over one year
inflowsHist = zeros(48,1);
rainfallHist = zeros(48,1);
snowHist = zeros(48,1);
for i=1:length(inflows)    
    inflowsHist(mod(i,48)+1) =  inflowsHist(mod(i,48)+1) + inflows(i);
    rainfallHist(mod(i,48)+1) =  rainfallHist(mod(i,48)+1) + rainfall(i);
    snowHist(mod(i,48)+1) =  snowHist(mod(i,48)+1) + snow(i);
end
inflowsHist = inflowsHist ./ floor(length(inflows) / 48);
rainfallHist = rainfallHist ./ floor(length(rainfall) / 48);
snowHist = snowHist ./ floor(length(snow) / 48);
x = 1:1:48;

%Inflows figure
figure('Name','Average inflow variations over one year')
plot(x, inflowsHist, 'DisplayName','inflows');
xlabel({'Period of the year - 48 measures each year starting on January 1st'});
ylabel({'Quantity'});
title({'Average inflow variations over one year'});
legend('inflows');

%Rainfall and snow figure
figure('Name','Average rainfall and snowmelt variations over one year')
plot(x, rainfallHist, 'b', x, snowHist, 'r');
% plot(x, snowHist, 'r', 'DisplayName','snow');
xlabel({'Period of the year - 48 measures each year starting on January 1st'});
ylabel({'Quantity'});
title({'Average rainfall and snowmelt variations over one year'});
legend('rainfall', 'snowmelt');

% Compute the Average Relative Variance (AVG)
inflowsAverageHist = zeros(1440, 1); % We start after the 1st year, otherwise results will be negatively biased
for i = 49:length(inflows)
    if floor(i/48) < 2
        inflowsAverageHist(i) = inflows(i-48);
    else
        inflowsAverageHist(i) = (inflowsAverageHist(i-48)*floor(i/48-2) + inflows(i-48))/floor(i/48-1);
    end  
end
numerator = ((sum((inflows(49:1440) - inflowsAverageHist(49:1440)) .^2)) / size(inflowsAverageHist(49:1440),1));
denominator = ((sum((inflows(49:1440) - mean(inflows(49:1440))) .^2)) / size(inflows(49:1440),1));
AverageHistARV = numerator / denominator

% AverageHist learning
figure('Name','Average inflow for previous years','NumberTitle','off')
x = 1953:1/48:1983-1/48; % We take 48 since we have 48 values for each year
plot(x,inflowsAverageHist, 'b')
hold on % When we set hold to on, MATLAB does not remove the existing graph;
        % it adds the new data to the current graph, rescaling if the new 
        % data falls outside the range of the previous axis limits. 
plot(x,inflows, 'r')
grid on 
legend(': Average inflow for previous year', 'Actual inflows');
hold off


%% Data standardization
% We want the transformed scores to have a mean of zero and a variance 
% (and standard deviation) of 1.

% inflows = inflows ./1000; % It turned out it wasn't such a good idea. But
                            % still a good approximation .
global inflowsMean; % We'll need it later.
global inflowsSTD ; % We'll need it later.
inflowsMean = mean(inflows);
inflowsSTD = mean(inflows);
inflows = (inflows-inflowsMean) ./ inflowsSTD;
rainfall = (rainfall-mean(rainfall)) ./ std(rainfall);
snow = (snow-mean(snow)) ./ std(snow);


%% Multilayer perceptron (MLP)

fprintf('\n\n********************************************************\n');
fprintf('Multilayer perceptron (MLP)\n');

% Import
warning off    % To avoid the 2 warnings that will appear during the import
               % 1) isscalar.m has the same name as a MATLAB builtin. We 
               %    suggest you rename the function to avoid a potential
               %    name conflict. 
               % 2) isvector.m has the same name as a MATLAB builtin. We 
               %    suggest you rename the function to avoid a potential
               %    name conflict. 
global options
addpath Netlab % Library of MATLAB functions for simulating neural network algorithms
               % http://www1.aston.ac.uk/eas/research/groups/ncrg/resources/netlab/

% NOTE : the Netlab library has been slightly customized for our needs: 
% Netlab/scg.m - line 155 has been commented so as to avoid too many outputs
% Netlab/scg.m - line 206 has been commented so as to avoid too many outputs             
               
warning on


% We set a few constants
DApp_size = 500;   % Learning set size    : de 1953  ??
DVal_size = 500;   % Validation set size  : de ??  ??
DTest_size = 440;  % Test set size        : de ??  1982
% We must have 1440 values, i.e. 4 * 12 value during (1983-1953 years)
assert(DApp_size + DVal_size + DTest_size == 1440, 'ERROR in Multilayer perceptron: DApp_size + DVal_size + DTest_size is not equal to 1440' ); 



% On itre sur le Predict_size, le nombre de valeurs utilises pour la 
% prdiction par historique.
for Predict_size = [1] % 6,12,48,2*48, 3*48, 4*48
    % Matrix allocations and definitions
    input = zeros(size(inflows,1) - Predict_size, Predict_size * 3); % Entres de toute la srie ;
    output = zeros(size(inflows,1) - Predict_size, 1);               % Sorties de toute la srie
    for i = 1:(size(inflows,1)-Predict_size)
        % First: we try to predict the inflow values using the %Predict_size%
        %        precedent inflow values. We therefore don't take into account the values
        %        of snow and rainfall.
        % input(i, :) = inflows(i:i+Predict_size-1,1)';
        
        % Second: we try to predict the inflow values using the %Predict_size%
        %         precedent inflow, rainfall and snow values. Unlike our first try, we now 
        %         take into account the values of snow and rainfall.
        input(i, :) = horzcat(rainfall(i:i+Predict_size-1,1)', snow(i:i+Predict_size-1,1)', inflows(i:i+Predict_size-1,1)'); % We concatenate inflows, rainfall and snow values
        % input(i, :) = horzcat(rainfall(i:i+Predict_size-1,1)', snow(i:i+Predict_size-1,1)');
        
        output(i)   = inflows(i+Predict_size, 1);
    end
    fprintf('\n Predict_size = %d \n', Predict_size)
    mlpWrapper(input, output, Predict_size, DApp_size, DVal_size, DTest_size )
end



%% Support Vector Regression (SVR)
% Lack of time....
% fprintf('\n\n********************************************************\n');
% fprintf('Support Vector Regression\n');
% 
% % Toolbox SVM-KM: http://asi.insa-rouen.fr/enseignants/~arakotom/toolbox
% % Add SVM-KM directories to MATLAB's search path.
% warning off    % To avoid the 2 warnings that will appear during the import
%                % 1) isscalar.m has the same name as a MATLAB builtin. We 
%                %    suggest you rename the function to avoid a potential
%                %    name conflict. 
%                % 2) isvector.m has the same name as a MATLAB builtin. We 
%                %    suggest you rename the function to avoid a potential
%                %    name conflict. 
% run('./addSVM_KM.m');
% warning on
% 
% % Learning
% % load datareg1
% C = 1; 
% lambda = 0.000001;
% epsilon=0.1;
% kernel='gaussian'; 
% kerneloption = 0.1;
% Predict_size = 0;
% X = 1953+(Predict_size*1/48):1/48:1983-1/48; % We take 48 since we have 48 values for each year
% X = X';
% [xsup,ysup,w,w0] = svmreg(X,inflows,C,epsilon,kernel,kerneloption,lambda);
% 
% xx=linspace(-2,2,10000)'; yy=sin(pi*xx).*((xx>-1) & (xx<1));
% rx = svmval(xx,xsup,w,w0,kernel,kerneloption);
% figure
% hold on
% h=plot(xx,yy,'g'); set(h,'LineWidth',2);
% h = plot(xx,rx,'b'); set(h,'LineWidth',2);
% h = plot(xsup,ysup,'or'); set(h,'LineWidth',2);
% plot(xx,rx+epsilon,'b-');
% plot(xx,rx-epsilon,'b-');
% title([ 'NbVectSupport = ' num2str(size(xsup,1))])

