%%%%%%% Self-organizing map (aka SOM aka Kohonen map) %%%%%%%%
%
% Subject: This file is part of the Project RCP209 @ CNAM Paris, 
%          France - Summer 2010
%          This files draws a Kohonen map to classify months as a function
%          of inflows, rainfall and snow.
% 
% To:  Michel CRUCIANU <Michel.Crucianu@cnam.fr>, 
%      Meziane YACOUB <meziane.yacoub@cnam.fr>
% 
% ----
% Franck DERNONCOURT <franck.dernoncourt@gmail.com>
% Phone: +33 601 188 949
% GTalk / Skype: franck.dernoncourt
% Address: 75013 Paris, France
% http://francky.me/
% http://www.linkedin.com/in/franckdernoncourt
% MATLAB : R2010a 64-bit (7.10.0.499)
% ----
%
% Data source: http://www.stats.uwo.ca/faculty/aim/epubs/mhsets/thompsto/
% Subject: 
% << 
% Prdiction  du dbit entrant d'un lac. L'objectif est de
% prdire l'volution du dbit entrant d'un lac  partir de
% l'historique de ce dbit, de la fonte des neiges et des prcipitations
% dans le bassin hydrographique.  Donnes :
% http://www.stats.uwo.ca/faculty/aim/epubs/mhsets/thompsto/
% Contact : Michel CRUCIANU <Michel.Crucianu@cnam.fr> 
% >>
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%% Data initialization %%
% D is the data we want to classify
load inflows
load snow
load rainfall

% Data standardization
% We want the transformed scores to have a mean of zero and a variance 
% (and standard deviation) of 1.
inflows = (inflows-mean(inflows)) ./ std(inflows);
rainfall = (rainfall-mean(rainfall)) ./ std(rainfall);
snow = (snow-mean(snow)) ./ std(snow);

D = [inflows, snow];
n = 1440;
% D Normalization
for i = 1:length(D)
    normV = sqrt(D(i,1)^2 + D(i,2)^2);
    D(i,1) = D(i,1) ./ normV;
    D(i,2) = D(i,2) ./ normV;
end

% cnames contains the name of each column
cnames = {'X', 'Y'};


% labs contains the initial classification
i1 = linspace(1,1440, 1440);
numberOfMonths = 12;         % Jan, Feb, March, ..., Nov, Dec.
numberOfMonthlyMeasures = 4; % 4 because there 4 measures each month
labs = cell(size(D,1),1);
for i = 1:length(i1) 
    labs{i1(i)} = ['mois ' int2str(mod((floor((i-1)/numberOfMonthlyMeasures)),numberOfMonths) + 1)]; 
end



%%%% LEARNING %%%%
%% Import somtoolbox library
% http://www.cis.hut.fi/somtoolbox/
addpath somtoolbox

% Data structure definition
sData = som_data_struct(D,'name','Cinq Gaussiennes','labels',labs,'comp_names',cnames);

% Type sData to visualize sData structure
% Type help som_data_struct for more information
% Initialisation de la structure de la carte topologique
msize = [6 6];
insize = size(sData.data,2);
lattice = 'rect'; % 'rect' ou 'hexa'
shape = 'sheet'; % 'sheet', 'cyl', ou 'toroid'
sMap = som_map_struct(insize,'msize',msize, lattice, shape);

% Tapez sData pour voir les diffrents champs de cette structure
% Tapez help som data struct pour plus de dtails
% Initialisation de la structure de la carte topologique
msize = [6 6];
insize = size(sData.data,2);
lattice = 'rect'; % 'rect' ou 'hexa'
shape = 'sheet'; % 'sheet', 'cyl', ou 'toroid'
sMap = som_map_struct(insize,'msize',msize, lattice, shape);

% Initialisation des poids de la carte topologique
sMap = som_randinit(sData, sMap); % som_randinit ou som_lininit

% Visualisation des donnees et de la carte initiale.
figure('Position',[100 100 500 500])
plot(D(:,1),D(:,2),'b+');
hold on
som_grid(sMap,'Coord',sMap.codebook)
axis on
title('Donnees et structure de la grille');

% Visualisation des donnees et de la carte initiale.
figure('Position',[100 100 500 500])
plot(D(:,1),D(:,2),'b+');
hold on
som_grid(sMap,'Coord',sMap.codebook)
axis on
title('Donnees et structure de la grille');

% Entranement de la carte : Phase 1 (Auto organisation)
figure('Position',[100 100 500 500])
epochs = 100;
radius_ini = 3;
radius_fin = 1;
Neigh = 'gaussian'; % 'gaussian', 'cutgauss', 'bubble' ou 'ep'
tr_lev = 3;
[sMap,sT] = som_batchtrain(sMap, sData,'trainlen', epochs, 'radius_ini', radius_ini, 'radius_fin', radius_fin, 'neigh', Neigh, 'tracking', tr_lev);
xlabel('AUTO ORGANISATION')

% Entranement de la carte : Phase 2 (Convergence)
epochs = 100;
radius_ini = 1;
radius_fin = 0.1;
figure('Position',[100 100 500 500])
[sMap,sT] = som_batchtrain(sMap, sData,'trainlen',epochs, 'radius ini',radius_ini,'radius_fin',radius_fin, 'neigh',Neigh,'tracking',tr_lev);
xlabel('CONVERGENCE')

% Visualisation des donnees et de la carte obtenue par apprentissage.
figure
plot(D(:,1),D(:,2),'b+')
hold on
som_grid(sMap,'Coord',sMap.codebook), hold off, axis on
title('Apres apprentissage en 2 phases : phase de convergence');
hold on
[Bmus, Qerrors] = som_bmus(sMap, sData);

% Donnees captees par une neurone particulier
neur=6
exemples_captes = find(Bmus == neur);
hold on
plot(D(exemples_captes,1),D(exemples_captes,2),'r+')
plot(sMap.codebook(neur,1),sMap.codebook(neur,2),'go')

% Qualit de la carte
[qe_conv,te_conv]=som_quality(sMap,sData)

% Labeliser les neurones de la carte en utilisant le vote majoritaire
sMap = som_label(sMap,'clear','all')
Labl = cell(size(sMap.codebook,1),1);
for ii = 1:size(sMap.codebook,1), Labl{ii} = num2str(ii); end
sMap = som_label(sMap,'add', 'all', Labl);
sMap = som_autolabel(sMap,sData,'vote');
figure('Position',[750 50 500 500]);
som_show(sMap,'empty','cell number and Labels');
som_show_add('label',sMap,'subplot',1);